# Copyright (c) Meta Platforms, Inc. and affiliates.
# All rights reserved.
#
# This source code is licensed under the BSD-style license found in the
# LICENSE file in the root directory of this source tree.

import logging

import torch

from torchvision import models

from ..model_base import EagerModelBase


class MV3Model(EagerModelBase):
    def __init__(self):
        pass

    def get_eager_model(self) -> torch.nn.Module:
        logging.info("Loading mobilenet_v3 model")
        # pyre-ignore
        mv3_small = models.mobilenet_v3_small(
            # pyre-ignore[16]
            weights=models.MobileNet_V3_Small_Weights.IMAGENET1K_V1
        )
        logging.info("Loaded mobilenet_v3 model")
        return mv3_small

    def get_example_inputs(self):
        tensor_size = (1, 3, 224, 224)
        return (torch.randn(tensor_size),)
